@@ -1,26 +1,26 @@
-/*  Rational - Rational number class with overflow detection
-    Copyright (C) 2012-2019 Antonio Diaz Diaz.
+/* Rational - Rational number class with overflow detection
+   Copyright (C) 2012-2020 Antonio Diaz Diaz.
 
-    This program is free software. Redistribution and use in source and
-    binary forms, with or without modification, are permitted provided
-    that the following conditions are met:
+   This program is free software. Redistribution and use in source and
+   binary forms, with or without modification, are permitted provided
+   that the following conditions are met:
 
-    1. Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
+   1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions, and the following disclaimer.
 
-    2. Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
+   2. Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions, and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 /*
-    Exit status: 0 for a normal exit, 1 for environmental problems
-    (file not found, invalid flags, I/O errors, etc), 2 to indicate a
-    corrupt or invalid input file, 3 for an internal consistency error
-    (eg, bug) which caused rational to panic.
+   Exit status: 0 for a normal exit, 1 for environmental problems
+   (file not found, invalid flags, I/O errors, etc), 2 to indicate a
+   corrupt or invalid input file, 3 for an internal consistency error
+   (eg, bug) which caused rational to panic.
 */
 
 #include <cctype>
@@ -38,8 +38,8 @@
 namespace {
 
 const char * const program_name = "rational";
-const char * const program_year = "2019";
-const char * invocation_name = 0;
+const char * const program_year = "2020";
+const char * invocation_name = program_name;		// default value
 
 
 void show_help()
@@ -49,36 +49,37 @@ void show_help()
                "\nRational allows you to perform exact calculations and verify at any time if\n"
                "overflow has occurred. Rational does not throw exceptions.\n"
                "\nIn case of domain error or overflow, denominator is set to 0 and numerator\n"
-               "is set to >0, <0 or 0, meaning +INF, -INF and NAN respectively. This error\n"
-               "condition can be tested with the 'error' function, and can only be cleared\n"
+               "is set to >0, <0, or 0, meaning +INF, -INF, and NAN respectively. This error\n"
+               "condition can be tested with the function 'error', and can only be cleared\n"
                "assigning a new value to the Rational.\n"
                "\nWhile in error state, arithmetic operators become no ops and relational\n"
                "operators return false, except '!=', which returns true. This is done to\n"
                "preserve the error condition which triggered the error, and because the\n"
                "error value is considered to be out of domain and therefore is different\n"
                "from any valid value, but not necessarily larger or smaller.\n"
-               "\nThe 'parse' function can be used to obtain values exactly representable by\n"
-               "small fractions like '0.1', '1/3' or '355/113' from the command line.\n"
-               "\nThe 'to_decimal' function prints values in decimal point form (123.456),\n"
-               "while the 'to_fraction' function prints values in fraction form (123/456).\n"
-               "Both functions print '+INF', '-INF' or 'NAN' to signal an error condition.\n"
+               "\nThe function 'parse' can be used to obtain values exactly representable by\n"
+               "small fractions like '0.1', '1/3', or '355/113' from the command line.\n"
+               "\nThe function 'to_decimal' prints values in decimal point form (123.456),\n"
+               "while the function 'to_fraction' prints values in fraction form (123/456).\n"
+               "Both functions print '+INF', '-INF', or 'NAN' to signal an error condition.\n"
                "\nValues are kept normalized (gcd(numerator,denominator) == 1, and the\n"
                "denominator is positive).\n"
                "Range extends from INT_MAX to -INT_MAX.\n"
                "Maximum resolution is 1 / INT_MAX.\n"
-               "\nTo use the Rational class in your own programs simply copy the files\n"
-               "'rational.h' and 'rational.cc' in your source tree. See also the file\n"
-               "'main.cc' for an example of use.\n"
+               "\nTo use the class Rational in your own programs simply copy the files\n"
+               "'rational.h' and 'rational.cc' in your source tree. See the file 'main.cc'\n"
+               "for an example of use.\n"
                "\nUsage: %s [options] [arguments]\n", invocation_name );
   std::printf( "\nOptions:\n"
-               "  -h, --help                   display this help and exit\n"
-               "  -V, --version                output version information and exit\n"
-               "  -p, --pi                     calculate approximations of pi\n"
-               "  -q, --quiet                  quiet operation\n"
-               "  -s, --sqrt=<n>               calculate approximations of sqrt(n)\n"
-               "  -t, --test=<level>           run tests at given level (1-9)\n"
-               "  -v, --verbose                verbose operation\n"
-               "The option -s, --sqrt accepts a rational argument.\n"
+               "  -h, --help                 display this help and exit\n"
+               "  -V, --version              output version information and exit\n"
+               "  -g, --golden-ratio         calculate approximations of (1 + sqrt(5)) / 2\n"
+               "  -p, --pi                   calculate approximations of pi\n"
+               "  -q, --quiet                quiet operation\n"
+               "  -s, --sqrt=<n>             calculate approximations of sqrt(n)\n"
+               "  -t, --test=<level>         run tests at given level (1-9)\n"
+               "  -v, --verbose              verbose operation\n"
+               "\nThe option '-s, --sqrt' accepts a rational argument.\n"
                "\nReport bugs to arg-parser-bug@nongnu.org\n"
                "Rational home page: http://www.nongnu.org/arg-parser/rational.html\n" );
   }
@@ -114,11 +115,45 @@ void internal_error( const char * const msg )
   }
 
 
+void approximate_golden_ratio()
+  {
+  const std::string target( "1.618033988749894848204586834365638117720" );
+  const int tlen = target.size();
+  int max_len = 1;		// number of chars identical to target
+  int n = 1, d = 1;	// Fibonacci F(n+1)/F(n) converges to the golden ratio
+
+  std::printf( "target = %s\n", target.c_str() );
+  std::fputs( "correct\ndecimals                  value\n", stdout );
+  while( true )
+    {
+    long long rest = n % d;
+    int i = 2;
+    for( ; i < tlen; ++i )
+      {
+      rest *= 10; const int dif = '0' + ( rest / d ) - target[i]; rest %= d;
+      if( dif != 0 ) break;
+      }
+    if( i > max_len )
+      {
+      const Rational r( n, d );
+      max_len = i;
+      std::printf( "%2d      %s (%s)\n", i - 2,
+                   r.to_decimal( 1, -(tlen - 2) ).c_str(),
+                   r.to_fraction().c_str() );
+      std::fflush( stdout );
+      if( max_len >= tlen ) break;
+      }
+    if( n > INT_MAX - d ) break;
+    const int next = n + d; d = n; n = next;
+    }
+  }
+
+
 void approximate_pi()
   {
   const std::string target( "3.141592653589793238462643383279502884197" );
   const int tlen = target.size();
-  int max_len = 1;
+  int max_len = 1;		// number of chars identical to target
   int n = 3, d = 1, dif = 0;
 
   std::printf( "target = %s\n", target.c_str() );
@@ -571,23 +606,25 @@ int main( const int argc, const char * const argv[] )
   {
   int level = 0;
   Rational sqrt_target;
+  bool gr_flag = false;
   bool pi_flag = false;
   bool verbose = false;
-  invocation_name = argv[0];
+  if( argc > 0 ) invocation_name = argv[0];
 
   const Arg_parser::Option options[] =
     {
-    { 'h', "help",    Arg_parser::no  },
-    { 'p', "pi",      Arg_parser::no  },
-    { 'q', "quiet",   Arg_parser::no  },
-    { 's', "sqrt",    Arg_parser::yes },
-    { 't', "test",    Arg_parser::yes },
-    { 'v', "verbose", Arg_parser::no  },
-    { 'V', "version", Arg_parser::no  },
-    {  0 , 0,         Arg_parser::no  } };
+    { 'g', "golden-ratio", Arg_parser::no  },
+    { 'h', "help",         Arg_parser::no  },
+    { 'p', "pi",           Arg_parser::no  },
+    { 'q', "quiet",        Arg_parser::no  },
+    { 's', "sqrt",         Arg_parser::yes },
+    { 't', "test",         Arg_parser::yes },
+    { 'v', "verbose",      Arg_parser::no  },
+    { 'V', "version",      Arg_parser::no  },
+    {  0 , 0,              Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options );
-  if( parser.error().size() )				// bad option
+  if( !parser.error().empty() )				// bad option
     { show_error( parser.error().c_str(), 0, true ); return 1; }
 
   int argind = 0;
@@ -598,6 +635,7 @@ int main( const int argc, const char * const argv[] )
     const char * const arg = parser.argument( argind ).c_str();
     switch( code )
       {
+      case 'g': gr_flag = true; break;
       case 'h': show_help(); return 0;
       case 'p': pi_flag = true; break;
       case 'q': verbose = false; break;
@@ -629,6 +667,7 @@ int main( const int argc, const char * const argv[] )
         retval = 1; }
     }
 
+  if( gr_flag ) approximate_golden_ratio();
   if( pi_flag ) approximate_pi();
   if( sqrt_target > 0 ) approximate_sqrt( sqrt_target );
   return retval;
